local addonName, addonTable = ...


--local Variables:
--Functions:

--Frames:

--Labels:

--Variables:
local equipped = {} --table containing all equipped items, indexed 1 through 18, except 4
local toReplace = {} --table containing the itemas that need replacing for each slot, indexed using localised slot names
local loot = {} --2D table containing items indexed with bossIndex and lootIndex
local lootBySlot = {} --table containing all the best loot items

--Constants	
local unusedStats = {
	ITEM_MOD_CR_STURDINESS_SHORT = 0,
	ITEM_MOD_CR_SPEED_SHORT = 0,
	ITEM_MOD_DAMAGE_PER_SECOND_SHORT = 0,
	EMPTY_SOCKET_WIND = 0,
	EMPTY_SOCKET_LIFE = 0,
	EMPTY_SOCKET_IRON = 0,
	EMPTY_SOCKET_FIRE = 0,
	EMPTY_SOCKET_FEL = 0,
	EMPTY_SOCKET_SHADOW = 0,
	EMPTY_SOCKET_HOLY = 0,
	EMPTY_SOCKET_BLOOD = 0,
	EMPTY_SOCKET_ARCANE = 0,
	EMPTY_SOCKET_FROST = 0,
	EMPTY_SOCKET_PRISMATIC = 0
}
local weaponLocToSlot = {
	INVTYPE_WEAPON = "Main-Hand",
	INVTYPE_SHIELD = "Off-Hand",
	INVTYPE_2HWEAPON = "Two-Hand",
	INVTYPE_WEAPONMAINHAND = "Main-Hand",
	INVTYPE_WEAPONOFFHAND = "Off-Hand",
	INVTYPE_HOLDABLE = "Off-Hand",
	INVTYPE_RANGED = "Two-Hand",
	INVTYPE_RANGEDRIGHT = "Two-Hand"
}
local weaponSlots = {
	"Main-Hand",
	"Off-Hand",
	"Two-Hand"
}
local gemStats = {
	--110
	[130215] = {"ITEM_MOD_CRIT_RATING_SHORT", 7 }, -- Deadly Deep Amber
	[130216] = {"ITEM_MOD_HASTE_RATING_SHORT", 7 }, -- Quick Azsunite
	[130217] = {"ITEM_MOD_VERSATILITY", 7 }, -- Versatile Skystone
	[130218] = {"ITEM_MOD_MASTERY_RATING_SHORT", 7 }, -- Masterful Queen's Opal
	[130219] = {"ITEM_MOD_CRIT_RATING_SHORT", 9 }, -- Deadly Eye of Prophecy
	[130220] = {"ITEM_MOD_HASTE_RATING_SHORT", 9 }, -- Quick Dawnlight
	[130221] = {"ITEM_MOD_VERSATILITY", 9 }, -- Versatile Maelstrom Sapphire
	[130222] = {"ITEM_MOD_MASTERY_RATING_SHORT", 9 }, -- Masterful Shadowruby
	[151580] = {"ITEM_MOD_CRIT_RATING_SHORT", 11 }, -- Deadly Deep Chemirine
	[151583] = {"ITEM_MOD_HASTE_RATING_SHORT", 11 }, -- Quick Lightsphene
	[151584] = {"ITEM_MOD_MASTERY_RATING_SHORT", 11 }, -- Masterful Argulite
	[151585] = {"ITEM_MOD_VERSATILITY", 11 }, -- Versatile Labradorite
	--120
	[153710] = {"ITEM_MOD_CRIT_RATING_SHORT", 30 }, -- Deadly Solstone
	[153711] = {"ITEM_MOD_HASTE_RATING_SHORT", 30 }, -- Quick Golden Beryl
	[153712] = {"ITEM_MOD_VERSATILITY", 30 }, -- Versatile Kyanite
	[153713] = {"ITEM_MOD_MASTERY_RATING_SHORT", 30 }, -- Masterful Kubiline
	[154126] = {"ITEM_MOD_CRIT_RATING_SHORT", 40 }, -- Deadly Amberblaze
	[154127] = {"ITEM_MOD_HASTE_RATING_SHORT", 40 }, -- Quick Owlseye
	[154128] = {"ITEM_MOD_VERSATILITY", 40 }, -- Versatile Royal Quartz
	[154129] = {"ITEM_MOD_MASTERY_RATING_SHORT", 40 }, -- Masterful Tidal Amethyst
	[153709] = {"ITEM_MOD_INTELLECT_SHORT", 40 },--Kraken's Eye of Intellect
	[153707] = {"ITEM_MOD_STRENGTH_SHORT", 40 },--Kraken's Eye of Strength
	[153708] = {"ITEM_MOD_AGILITY_SHORT", 40 },--Kraken's Eye of Agility
}

--Creating the Item class
local Item = {}
Item.mt = {}
Item.mt.__index = Item

--calculates the score of an item
function Item:calcScore()
	if not self then
		return
	end
	self.score = 0

	--adds stats from gems
	if self.stats["EMPTY_SOCKET_PRISMATIC"] then
		local _, _, Color, Ltype, Id, Enchant, Gem1, Gem2, Gem3, Gem4, Suffix, Unique, LinkLvl, Name = string.find(self.link, "|?c?f?f?(%x*)|?H?([^:]*):?(%d+):?(%d*):?(%d*):?(%d*):?(%d*):?(%d*):?(%-?%d*):?(%-?%d*):?(%d*):?(%d*):?(%-?%d*)|?h?%[?([^%[%]]*)%]?|?h?|?r?")
		local gems = {tonumber(Gem1), tonumber(Gem2), tonumber(Gem3), tonumber(Gem4)}
		for i, gem in ipairs(gems) do
			if gemStats[gem] then
				self.stats[gemStats[gem][1]] = self.stats[gemStats[gem][1]] + gemStats[gem][2]
			end
		end
	end

	--calculates base score from item stats
	for stat, value in pairs(self.stats) do
		self.score = self.score + ((brhV.statweights[brhV.playerInfo.specID][stat]
					or unusedStats[stat]
					or (print(brhT.text.error.unknownStat:format(self.name, stat)) or 0)) * value)
	end

	--adds bonus score based on name
	if brhV.bonusScores[self.name] then
		self.score = self.score + loadstring(brhV.bonusScores[self.name].func)() or 0
	end

	--adds bonus score based on azerite traits
	if self.azeritePowers then
		for i, tier in ipairs(self.azeritePowers) do
			local highestScore = 0
			for i, powerID in ipairs(tier.azeritePowerIDs) do
				local powerName = GetSpellInfo(C_AzeriteEmpoweredItem.GetPowerInfo(powerID).spellID)
				if brhV.bonusScores[powerName] then
					highestScore = max(highestScore, loadstring(brhV.bonusScores[powerName].func)() or 0)
				end
			end
			if brhV.options.useCurrentAzeriteLevel and tier.unlockLevel > (brhV.playerInfo.currentAzeriteLevel or 0) then
				highestScore = 0
			end
			self.score = self.score + highestScore
		end
	end

	return self.score
end

--returns a copy of an item
function Item:copy()
	local t = {}
	setmetatable(t, getmetatable(self))
	for k, v in pairs(self) do
		t[k] = v
	end
	return t
end

--returns a new item
function Item.new(link)
	link = link or ""
	local item = {}
	setmetatable(item, Item.mt)
	if not GetItemInfo(link) then
		item.link = ""
		item.slot = ""
		item.name = ""
		item.ID = 0
		item.bossIndex = -1
		item.stats = {}
		item.azeritePowers = false
		item.score = -1
		return item
	end
	local name, _, _, _, _, itemType, _, _, itemLoc, _, _, _, _, _, _, _, _ = GetItemInfo(link)
	--      1   2  3  4  5  6         7  8      9    10 11 12 13 14  15 16 17
	item.link = link
	item.slot = weaponLocToSlot[itemLoc] or _G[itemLoc]
	item.name = name
	item.ID = tonumber(link:match("item:%d+:"):match("%d+"))
	item.bossIndex = -1
	item.stats = {}
	item.azeritePowers = C_AzeriteEmpoweredItem.GetAllTierInfoByItemID(item.ID)
	if not next(item.azeritePowers) then
		item.azeritePowers = false
	end
	GetItemStats(link, item.stats)
	item.score = item:calcScore()
	return item
end

--Returns formatted BiS list
function brhT.printBiS()
	local output = ""
	for i = INVSLOT_FIRST_EQUIPPED, 18, 1 do
		if i ~= INVSLOT_NECK then
			local i_ = i ~= INVSLOT_BODY and i or INVSLOT_BACK
			local currItem = brhV.BiS[i_]
			if currItem.name ~= "" then
				if currItem.bossIndex > 0 then
					output = output..brhT.text.BiSOutput.lineLoot:format(currItem.slot, brhT.text.BiSOutput.colorLoot, currItem.name, EJ_GetEncounterInfoByIndex(currItem.bossIndex or 0 , brhT.raidID))
				else
					output = output..brhT.text.BiSOutput.lineEquipped:format(currItem.slot, brhT.text.BiSOutput.colorEquipped, currItem.name)
				end
			end
		end
	end
	return output
end

--calculates score of a specified item and returns its score
function brhT.calcItemScore(link)
	local item  = {}
	item = Item.new(link)
	return item.score, item
end

--function called by the UI button
function brhT.calc()
--Equipped
	wipe(equipped)
	wipe(toReplace)

	--Fills indexes 1 through 18 (except Shirt and Neck) with standard item info
	for i = INVSLOT_FIRST_EQUIPPED, 15, 1 do
		if i ~= INVSLOT_BODY and i ~= INVSLOT_NECK then
			equipped[i] = Item.new(GetInventoryItemLink("player", i) or "", i)
		end
	end
	local mainWeapon = Item.new(GetInventoryItemLink("player", 16) or "", 16)
	local offWeapon = Item.new(GetInventoryItemLink("player", 17) or "", 17)
	if mainWeapon.slot == weaponSlots[3] then
		equipped[16] = Item.new("")
		equipped[16].slot = weaponSlots[1]
		equipped[17] = Item.new("")
		equipped[17].slot = weaponSlots[2]
		equipped[18] = mainWeapon:copy()
	else
		equipped[16] = mainWeapon:copy()
		equipped[17] = offWeapon:copy()
		equipped[18] = Item.new("")
		equipped[18].slot = weaponSlots[3]
	end

	--Fills toReplace
	for i, item in pairs(equipped) do
		if not toReplace[item.slot] then
			toReplace[item.slot] = {}
		end
		tinsert(toReplace[item.slot], item:copy())
	end
	for slot, itemList in pairs(toReplace) do
		table.sort(itemList,
			function(i1, i2)
				return i1.score < i2.score
			end)
	end
--Loot
	wipe(loot)
	wipe(lootBySlot)
	for i = 1, brhT.numBosses, 1 do
		local currBoss = brhV.bosses[i]
		wipe(currBoss.loot)
		currBoss.score = 0
		currBoss.rank = 0
		currBoss.numLoot = 0
		currBoss.isKilled = false
		currBoss.shouldCoin = false
	end
	brhT.getPlayerInfo()
	EJ_SetLootFilter(brhV.playerInfo.classID, brhV.playerInfo.specID)
	EncounterJournalEncounterFrameInfoLootScrollFrameSlotFilterToggle:Click("LeftButton")
	DropDownList1Button1:Click("LeftButton")
	EJ_SelectInstance(brhT.raidID)

	--Fills loot with items
	for bossIndex = 1, brhT.numBosses, 1 do
		local currBoss = brhV.bosses[bossIndex]
		if currBoss.diff > 0 then
			EJ_SetDifficulty(currBoss.diff)
			EJ_SelectEncounter(currBoss.ID)
			loot[bossIndex] = {}
			currBoss.loot = loot[bossIndex]
			currBoss.numLoot = EJ_GetNumLoot()
			for lootIndex = 1, currBoss.numLoot, 1 do
				local realLootIndex = #loot[bossIndex] + 1
				local itemID, _, _, _, slot, _, link = EJ_GetLootInfoByIndex(lootIndex)
				if not link then
					print(brhT.text.error.notLoaded)
					return false
				end

				if slot ~= "" then
					loot[bossIndex][realLootIndex] = Item.new(link)
					loot[bossIndex][realLootIndex].bossIndex = bossIndex
				else
					currBoss.numLoot = currBoss.numLoot - 1
				end
			end
		else
			currBoss.numLoot = 0
		end		
	end

	--Fills lootBySlot with items
	for bossIndex, bossLoot in pairs(loot) do
		for lootIndex, currLoot in ipairs(bossLoot) do
			if not lootBySlot[currLoot.slot] then
				lootBySlot[currLoot.slot] = {}
			end
			tinsert(lootBySlot[currLoot.slot], currLoot:copy())
		end
	end
	--Sorts lootBySlot so that the best item is first
	for slot, t in pairs(lootBySlot) do
		table.sort(lootBySlot[slot],
		function(i1, i2)
			return i1.score > i2.score
		end)
	end
--BiS
	wipe(brhV.BiS)
	local allItems = {}
	for slot, itemList in pairs(toReplace) do
		allItems[slot] = {}
		for index, item in pairs(itemList) do
			tinsert(allItems[slot], item:copy())
		end
		if lootBySlot[slot] then
			for index, item in pairs(lootBySlot[slot]) do
				tinsert(allItems[slot], item:copy())
			end
		end
		table.sort(allItems[slot],
			function(i1, i2)
				return i1.score > i2.score
			end)
		local firstIndexWithScore, score = 1, 0
		for index, item in ipairs(allItems[slot]) do
			if item.score ~= score then
				item.score = score
				firstIndexWithScore = index
			else
				if item.bossIndex == -1 then
					for j = firstIndexWithScore, index, 1 do
						if allItems[slot][j].bossIndex ~= -1 then
							local tempItem = allItems[slot][j].bossIndex
							allItems[slot][j] = allItems[slot][index]
							allItems[slot][index] = tempItem
							break
						end
					end
				end
			end
		end
	end
	for i = INVSLOT_FIRST_EQUIPPED, 18, 1 do
		if i ~= INVSLOT_BODY and i ~= INVSLOT_NECK then --Shirt
			local k = 1	--we want the best item for each slot
			if i == INVSLOT_FINGER2 or i == INVSLOT_TRINKET2 then	--for trinkets/rings we need the second best aswell
				if allItems[equipped[i].slot][2].name == allItems[equipped[i].slot][1].name then --if the second best item has the same name as the best, we can't equipp both
					k = 3
				else
					k = 2
				end
			end
			brhV.BiS[i] = allItems[equipped[i].slot][k]:copy()
		end
	end
--Boss scores
	for i, currBoss in ipairs(brhV.bosses) do
		if currBoss.diff == 0 then
			currBoss.score = -1
		end
	end
	for bossIndex, bossLoot in pairs(loot) do
		local currBoss = brhV.bosses[bossIndex]
		for lootIndex, item in ipairs(bossLoot) do
			if toReplace[item.slot] and toReplace[item.slot][1].name ~="" then
				currBoss.score = currBoss.score + max(0, item.score - toReplace[item.slot][1].score)
			end
		end
		currBoss.score = currBoss.score / currBoss.numLoot
	end
--Boss rankings
	for i = 1, brhT.numBosses, 1 do
		brhV.bosses[i].rank = 0
	end


	for i = 1, brhT.numBosses, 1 do
		local bestBoss = brhT.newBoss()
		bestBoss.score = -2
		for j = 1, brhT.numBosses, 1 do
			local currBoss = brhV.bosses[j]
			if currBoss.rank == 0 and currBoss.score > bestBoss.score then
				bestBoss = currBoss
			end
		end
		bestBoss.rank = i
	end
end