local addonName, addonTable = ...


--local Variables:

--Functions:
local createMainFrame		--Function creating the mainFrame
local setCalculatedBossInfo

--Frames:
local mainFrame				--The big Frame cointaining all UI elements
local closeButton			--The "X" button at the top right
local resultButton = {}		--Table containing the EJ-style buttons displaying calculation results
local bossCheckBoxes = {}	--2D table containing all checkboxes for selecting bosses you expect to kill
local calcButton			--Button triggering the calculation
local statWeightBoxes = {}	--Table containing all statweight boxes
brhT.statWeightBoxes = statWeightBoxes
local BiSOutput				--EditBox containing the BiS list
local BiSButton				--Button showing BiSOutput
local bonusScoreSelect		--Dropdown selecting an item name
local bonusScoreValue		--EditBox for inputting a value/formula
local bonusScoreNew			--Editbox for entering a new item name
local optionBoxes = {}		--Table containing checkboxes for the options
local eventFrame			--Frame listening to events
local importStatweightsButton	--Button to show the import statweights EdiBox
local importStatweightsEditBox	--EditBox to paste the import string into
local difficultyLabelsFrames = {}
local pawnImportButton
local pawnImpotEditbox

--Labels:
local statweightsLabel		--The big label above all statweights
local statLabel = {}		--Table containing the labels for each statweight box
local difficultyLabels = {}	--Table containing the Labels for each difficulty
local bossSelectLabel		--The big label above the boss select checkboxes

--Variables:
local bonusScoreCurrentName
local biSIsUpdate = false


--Sets up Slash Commands
SLASH_BONUSROLLHELPER1, SLASH_BONUSROLLHELPER2 = "/bonusrollhelper", "/brh"
function SlashCmdList.BONUSROLLHELPER(msg, editbox)
	setmetatable(brhV.statweights, brhT.statweightsMT)
	if msg == "" then
		if not brhT.frameCreated then
			brhT.frameCreated = true
			createMainFrame()
			if brhT.didVersionChange then
				if brhT.didVersionChange then
					local updateMessage = brhT.text.updateMessage
					for i, versionUpdateText in pairs(brhT.currChangeInfo) do
						updateMessage = updateMessage.."\n"..versionUpdateText
					end
					
					biSIsUpdate = true
					BiSOutput.EditBox:SetText(updateMessage)
					BiSOutput:Show()
					for i, button in ipairs(resultButton) do
						button:Hide()
					end
					BiSButton:SetText(brhT.text.BiSButton.hideUpdateText)
				end
			end
		elseif mainFrame:IsShown() then
			closeButton:Click()
		else
			BonusRollHelperMain:SetParent(UIParent)
			BonusRollHelperMain:Show()
			optionBoxes[1]:Click()
			optionBoxes[1]:Click()
			BiSButton:Click()
			BiSButton:Click()
		end
	elseif msg:find("item ") == 1 then
		local link = msg:gsub("item ", "")
		local score, item = brhT.calcItemScore(link)
		print(score)
	elseif msg == "show" then
		if GetSpellConfirmationPromptsInfo()[1] then
			BonusRollFrame:Show()
		else
			print(brhT.text.error.noBonusRollsHidden)
		end
	else
		print(brhT.text.error.unknownSlashCommand)
	end
end

--Local Functions:

--Is called when new recommended bosses are colculated. Puts their info into the calclated boss buttons.
setCalculatedBossInfo =
	function()
		local numSeals = select(2, GetCurrencyInfo(brhT.sealCurrencyID))
		local killedBosses = 0
		local raidName = EJ_GetInstanceInfo(brhT.raidID)

		for i = 1, GetNumSavedInstances(), 1 do
			local name, id, reset, difficulty, locked, extended, instanceIDMostSig, isRaid, maxPlayers, difficultyName, numEncounters, encounterProgress = GetSavedInstanceInfo(i)
			if name == raidName then
				for j = 1, numEncounters, 1 do
					if brhV.bosses[j].diff == difficulty then
						brhV.bosses[j].isKilled = select(3, GetSavedInstanceEncounterInfo(i, j))
					end
				end
			end
		end
		
		local bossByRanking = {}
		for i = 1, brhT.numBosses, 1 do
			for j = 1, brhT.numBosses, 1 do
				if brhV.bosses[j].rank == i then
					tinsert(bossByRanking, brhV.bosses[j])
				end
			end
		end

		for i = 1, 8, 1 do
			local currBoss = bossByRanking[i]
			local currButton = resultButton[i]
			currButton:Show()
			if currBoss then
				currButton.creature:SetDesaturated(false)
				currButton.link = currBoss.link
				currButton.diff = currBoss.diff
				currButton.boss = currBoss
				local color = ""
				if currBoss.isKilled then
					color = "|cFF808080"
					currButton.creature:SetDesaturated(true)
					killedBosses = killedBosses + 1
				elseif i - killedBosses <= numSeals then
					currBoss.shouldCoin = true
					color = "|cFF00FC00"
				end
				currButton:SetText(color..currBoss.name.."|r")
				local _, _, _, _, bossImage = EJ_GetCreatureInfo(1, currBoss.ID)
				bossImage = bossImage or "Interface\\EncounterJournal\\UI-EJ-BOSS-Default"
				currButton.creature:SetTexture(bossImage)
				
				if currBoss.diff == 0 then
					currButton:Hide()
				else
					currButton:Show()
				end
			else
				currButton.creature:SetTexture("Interface\\EncounterJournal\\UI-EJ-BOSS-Default")
			end
		end
	end
--Updates brhV
local function updateVariables()
	local currVersionChange = false
	if type(brhV.version) ~= "table" then
		brhT.createVariables()
		setmetatable(brhV.statweights, brhT.statweightsMT)
		brhT.didVersionChange = true
		currVersionChange = true
		tinsert(brhT.currChangeInfo, brhT.text.changeInfo[noVersion])
		brhV.version = brhT.version
	elseif brhV.version.major == 0 then
		if brhV.version.minor == 0 then
			for specID, statweights in pairs(brhV.statweights) do
				brhV.statweights[specID].EMPTY_SOCKET_PRISMATIC = nil
			end
			brhV.version.minor = 1
			brhT.didVersionChange = true
			currVersionChange = true
			tinsert(brhT.currChangeInfo, brhT.text.changeInfo[0][1])
		end
	end


	if currVersionChange then
		updateVariables()
	end
end

--Opens and creates the big frame
createMainFrame = function()
--References to the global Tables
	setmetatable(brhV.statweights, brhT.statweightsMT)

--Loads the Encounter Journal
	local loaded, reason = LoadAddOn("Blizzard_EncounterJournal")
	if not loaded then
		print(brhT.text.error.EJNotLoaded:format(tostring(reason)))
	end
	EJMicroButton:Click("LeftButton", true)
	EJMicroButton:Click("LeftButton", true)
	
	
	EncounterJournal_OpenJournal(15, brhT.raidID)
	EncounterJournalCloseButton:Click()
	
	brhT.getPlayerInfo()
	
	for i = 1, brhT.numBosses, 1 do
		if not brhV.bosses[i] then
			brhV.bosses[i] = brhT.newBoss()
		end
		brhV.bosses[i].name, _, brhV.bosses[i].ID, _, brhV.bosses[i].link = EJ_GetEncounterInfoByIndex(i, brhT.raidID)
		brhV.bosses[i].index = i
	end
	
--Sets up the main frame
 	mainFrame = CreateFrame("Frame", addonName.."Main", UIParent)
 	mainFrame:SetBackdrop({
    	bgFile="Interface\\DialogFrame\\UI-DialogBox-Background", 
    	edgeFile="Interface\\DialogFrame\\UI-DialogBox-Border", 
    	tile=1, tileSize=32, edgeSize=32, 
		insets={left=12, right=12, top=12, bottom=12}
	})
 	mainFrame:SetSize(unpack(brhT.graphics.mainFrame.size))
 	mainFrame:SetPoint("CENTER", UIParent, "CENTER",unpack(brhT.graphics.mainFrame.point))
 	mainFrame:EnableMouse(true)
 	mainFrame:EnableKeyboard(true)
 	mainFrame:SetMovable(true)
 	mainFrame:RegisterForDrag("LeftButton")
	mainFrame:SetScript("OnDragStart", mainFrame.StartMoving)
	mainFrame:SetScript("OnDragStop", mainFrame.StopMovingOrSizing)
 	mainFrame:SetFrameStrata("MEDIUM")
 	tinsert(UISpecialFrames, mainFrame:GetName())
	

--Sets up the close button on the top right
 	closeButton = CreateFrame("Button",addonName.."Close", mainFrame, "UIPanelCloseButton")
 	closeButton:SetSize(unpack(brhT.graphics.closeButton.size))
 	closeButton:SetPoint("TOPRIGHT", mainFrame, "TOPRIGHT", unpack(brhT.graphics.closeButton.point))
	closeButton:SetScript("OnClick",
		function(self) self:GetParent():Hide() self:GetParent():SetParent(nil)
	end)
	
--Sets up the statweight editboxes and their labels
	--Saves the input.
	local function setStatweights(self)
		if self:GetText() == "" then
			self:SetText(brhT.text.statWeightBoxes.text)
		end
		local number = tonumber(self:GetText())
		brhV.statweights[brhV.playerInfo.specID][self.stat.variable] = number
		self:SetText(number)
		self:HighlightText(0,0)
		self:ClearFocus()
		brhT.options[1].func(brhV.options.compactStatweights)
	end

	--Saves the imput and moves the curser to the next Statweight.
	local function setStatweightsTab(self)
		setStatweights(self)

		local offset = 0
		if IsShiftKeyDown() then
			offset = -1
		else
			offset = 1
		end

		for i = 1, 20, 1 do
			if not statWeightBoxes[self.index + offset] then
				break
			end

			if statWeightBoxes[self.index + offset]:IsShown() then
				statWeightBoxes[self.index + offset]:SetFocus()
				statWeightBoxes[self.index + offset]:HighlightText()
				break
			else
				offset = offset > 0 and offset + 1 or offset - 1
			end
		end
	end

 	for i, stat in ipairs(brhT.stats) do
		statWeightBoxes[i] = CreateFrame("EditBox", addonName.."statWeightBox"..i, mainFrame, "InputBoxTemplate")
		local currBox = statWeightBoxes[i]
		if i == 1 then
			currBox:SetPoint("TOPLEFT", mainFrame, "TOPLEFT", unpack(brhT.graphics.statWeightBoxes.pointInit))
		else
			currBox:SetPoint("TOPLEFT", statWeightBoxes[i - 1], "TOPLEFT", unpack(brhT.graphics.statWeightBoxes.pointGeneric))
		end
 		currBox:SetSize(unpack(brhT.graphics.statWeightBoxes.size))
 		currBox:SetFontObject(GameFontNormal)
		currBox:SetNumeric(false)
		currBox:SetMaxLetters(5)
 		currBox:SetText(brhV.statweights[brhV.playerInfo.specID][stat.variable] or brhT.text.statWeightBoxes.text)
		currBox:SetAutoFocus(false)
		currBox.index = i
		currBox.stat = stat
 		currBox:SetScript("OnEditFocusLost", setStatweights)
		currBox:SetScript("OnEnterPressed", setStatweights)
		currBox:SetScript("OnTabPressed", setStatweightsTab)
 				
		statLabel[i] = statWeightBoxes[i]:CreateFontString(addonName.."statWeightBox"..i.."Label", "OVERLAY", "GameFontNormal")
		statLabel[i]:SetSize(unpack(brhT.graphics.statWeightBoxes.labelSize))
		statLabel[i]:SetPoint("TOPRIGHT", statWeightBoxes[i], "TOPLEFT", unpack(brhT.graphics.statWeightBoxes.labelPoint))
		statLabel[i]:SetJustifyH("CENTER")
		statLabel[i]:SetText(stat.name)		
	end
--Sets up the Pawn import Button
	pawnImportButton = CreateFrame("Button", addonName.."pawnImportButton", mainFrame, "UIPanelButtonTemplate")
	pawnImportButton:SetSize(unpack(brhT.graphics.pawnImportButton.size))
	pawnImportButton:SetPoint("TOPRIGHT", statWeightBoxes[#statWeightBoxes], "TOPRIGHT", unpack(brhT.graphics.pawnImportButton.point))
	pawnImportButton:SetText(brhT.text.pawnImportButton.text)
	pawnImportButton:SetScript("OnClick", 
		function(self)
			self:Hide()
			pawnImpotEditbox:Show()
			pawnImpotEditbox:SetFocus()
			pawnImpotEditbox:HighlightText()
		end)
	--tooltip
	pawnImportButton:SetScript("OnEnter",
	function(self)
		GameTooltip:SetOwner(self, "ANCHOR_TOPLEFT")
		GameTooltip:SetText(brhT.text.pawnImportButton.tooltip, nil, nil, nil, nil, true)
		if not brhV.options.showTooltip then GameTooltip:Hide() end
	end)
	pawnImportButton:SetScript("OnLeave", function(self) GameTooltip:Hide() end)
--Sets up the Pawn iport Editbox
	pawnImpotEditbox = CreateFrame("EditBox", addonName.."pawnImpotEditbox", mainFrame, "InputBoxTemplate")
	pawnImpotEditbox:SetPoint("TOPLEFT", pawnImportButton, "TOPLEFT", unpack(brhT.graphics.pawnImpotEditbox.point))
	pawnImpotEditbox:SetSize(unpack(brhT.graphics.pawnImpotEditbox.size))
	pawnImpotEditbox:SetText(brhT.text.pawnImpotEditbox.text)
	pawnImpotEditbox:SetAutoFocus(false)
	local function importPawnString(pawnString)
		-- Read the scale and perform basic validation.
		local Pos, _, Version, Name, ValuesString = strfind(pawnString, "^%s*%(%s*Pawn%s*:%s*v(%d+)%s*:%s*\"([^\"]+)\"%s*:%s*(.+)%s*%)%s*$")
		Version = tonumber(Version)
		if (not Pos) or (not Version) or (not Name) or (Name == "") or (not ValuesString) or (ValuesString == "") then
			return
		end
		
		-- Now, parse the values string for stat names and values.
		local Values = {}
		local function SplitStatValuePair(Pair)
			local Pos, _, Stat, Value = strfind(Pair, "^%s*([%a%d]+)%s*=%s*(%-?[%d%.a-zA-Z]+)%s*,$")
			if Stat == "Class" then
				Stat = ""
				Value = nil
			elseif Stat == "Spec" then
				Stat = ""
				Value = nil
			else
				Value = tonumber(Value)
			end
			if Pos and Stat and (Stat ~= "") and Value then 
				Values[Stat] = Value
			end
		end
		gsub(ValuesString .. ",", "[^,]*,", SplitStatValuePair)
		
		-- Looks like everything worked.

		local pawnToVariable = {
			Strength = "ITEM_MOD_STRENGTH_SHORT",
			Agility = "ITEM_MOD_AGILITY_SHORT",
			Intellect = "ITEM_MOD_INTELLECT_SHORT",
			Stamina = "ITEM_MOD_STAMINA_SHORT",
			Armor = "RESISTANCE0_NAME",
			CritRating = "ITEM_MOD_CRIT_RATING_SHORT",
			HasteRating = "ITEM_MOD_HASTE_RATING_SHORT",
			MasteryRating = "ITEM_MOD_MASTERY_RATING_SHORT",
			Versatility = "ITEM_MOD_VERSATILITY",
			Avoidance = "ITEM_MOD_CR_AVOIDANCE_SHORT",
			Leech = "ITEM_MOD_CR_LIFESTEAL_SHORT",
			Dps = "ITEM_MOD_DAMAGE_PER_SECOND_SHORT",
		}
		local variableToPawn = {}
		for k, v in pairs(pawnToVariable) do
			variableToPawn[v] = k
		end

		local currStatWeights = brhV.statweights[brhV.playerInfo.specID]
		for variable, weight in pairs(currStatWeights) do
			currStatWeights[variable] = Values[variableToPawn[variable] or ""] or 0
		end

		brhV.test = {Name, Values}
	end
	local function pawnImportEditboxFunc(self)
		importPawnString(self:GetText())
		self:SetText("")
		self:HighlightText(0,0)
		self:ClearFocus()
		self:Hide()
		pawnImportButton:Show()

		for i, currBox in ipairs(statWeightBoxes) do
			currBox:SetText(brhV.statweights[brhV.playerInfo.specID][currBox.stat.variable] or brhT.text.statWeightBoxes.text)
		end
		optionBoxes[1]:Click()
		optionBoxes[1]:Click()
	end
	pawnImpotEditbox:SetScript("OnEditFocusLost", pawnImportEditboxFunc)
	pawnImpotEditbox:SetScript("OnEnterPressed", pawnImportEditboxFunc)
	pawnImpotEditbox:SetScript("OnTabPressed", pawnImportEditboxFunc)
	pawnImpotEditbox:Hide()

--Sets up the "Stat Weights:" label
	statweightsLabel = mainFrame:CreateFontString(addonName.."StatweightsLabel", "OVERLAY", "GameFontNormal")
	statweightsLabel:SetSize(unpack(brhT.graphics.statWeightBoxes.statWeightsLabelSize))
	statweightsLabel:SetPoint("BOTTOMRIGHT", statWeightBoxes[1], "TOPLEFT", unpack(brhT.graphics.statWeightBoxes.statWeightsLabelPoint))
	statweightsLabel:SetFont("Fonts\\FRIZQT__.TTF", 15, "")
	statweightsLabel:SetText(brhT.text.statWeightBoxes.label)
	statweightsLabel:SetJustifyH("RIGHT")
		
--Sets up Checkboxes and labels for Bosses
	for i = 1, brhT.numBosses, 1 do
		bossCheckBoxes[i] = {}
		for j = 1, 3, 1 do
			bossCheckBoxes[i][j] = CreateFrame("CheckButton", addonName.."bossCheckBoxes"..i.."_"..j, mainFrame, "UICheckButtonTemplate")
			local currBox = bossCheckBoxes[i][j]
			currBox:SetSize(unpack(brhT.graphics.bossCheckBoxes.size))
			if i == 1 and j == 1 then
				currBox:SetPoint("TOPLEFT", mainFrame, "TOPLEFT", unpack(brhT.graphics.bossCheckBoxes.pointInit))
			elseif j == 1 then
				currBox:SetPoint("TOPLEFT", bossCheckBoxes[i - 1][1], "TOPLEFT", unpack(brhT.graphics.bossCheckBoxes.pointInitRow))
			else
				currBox:SetPoint("TOPLEFT", bossCheckBoxes[i][j - 1], "TOPLEFT", unpack(brhT.graphics.bossCheckBoxes.pointGeneric))	
			end
			currBox.bossIndex = i
			currBox.diff = j + 13
			currBox.boss = brhV.bosses[i]
			currBox:SetScript("OnClick",
			function(self)
				if self:GetChecked() then
					self.boss.diff = self.diff
					bossCheckBoxes[self.bossIndex][1]:SetChecked(false)
					bossCheckBoxes[self.bossIndex][2]:SetChecked(false)
					bossCheckBoxes[self.bossIndex][3]:SetChecked(false)
					bossCheckBoxes[self.bossIndex][self.diff - 13]:SetChecked(true)
				else
					self.boss.diff = 0
					self.boss.score = -1
				end
			end)
			if brhV.bosses[i].diff - 13 == j then
				currBox:SetChecked(true)
			end
		end
		local currLabel = _G[bossCheckBoxes[i][1]:GetName().."Text"]
		currLabel:SetText(EJ_GetEncounterInfoByIndex(i, brhT.raidID))
		currLabel:ClearAllPoints()
		currLabel:SetPoint("TOPRIGHT", bossCheckBoxes[i][1], "TOPLEFT", unpack(brhT.graphics.bossCheckBoxes.labelPoint))
		currLabel:SetJustifyH("RIGHT")
		currLabel:SetSize(unpack(brhT.graphics.bossCheckBoxes.labelSize))
	end
	local difficultiesAlignment = {"RIGHT", "", "LEFT"}
	for i = 1, 3, 1 do
		difficultyLabelsFrames[i] = CreateFrame("Button", addonName.."DifficultyLabelButton"..i, mainFrame)
		difficultyLabelsFrames[i]:SetSize(unpack(brhT.graphics.bossCheckBoxes.diffLabelSize))
		difficultyLabelsFrames[i]:SetPoint("BOTTOM"..difficultiesAlignment[i], bossCheckBoxes[1][i], "TOP"..difficultiesAlignment[i], unpack(brhT.graphics.bossCheckBoxes.diffLabelPoint))
		difficultyLabelsFrames[i]:SetScript("OnMouseDown",
			function(self, button)
				if button == "LeftButton" then
					for j, checkBoxArray in ipairs(bossCheckBoxes) do
						checkBoxArray[i]:SetChecked(false)
						checkBoxArray[i]:Click()
					end
				elseif button == "RightButton" then
					for j, checkBoxArray in ipairs(bossCheckBoxes) do
						checkBoxArray[i]:SetChecked(true)
						checkBoxArray[i]:Click()
					end
				end
			end)
		difficultyLabels[i] = difficultyLabelsFrames[i]:CreateFontString(addonName.."DifficultyLabel"..i, "OVERLAY", "GameFontNormal")
		difficultyLabels[i]:SetSize(unpack(brhT.graphics.bossCheckBoxes.diffLabelSize))
		difficultyLabels[i]:SetPoint("BOTTOM"..difficultiesAlignment[i], bossCheckBoxes[1][i], "TOP"..difficultiesAlignment[i], unpack(brhT.graphics.bossCheckBoxes.diffLabelPoint))
		difficultyLabels[i]:SetText(GetDifficultyInfo(13 + i))
	end
	bossSelectLabel = bossCheckBoxes[1][1]:CreateFontString(addonName.."bossSelectLabel", "OVERLAY", "GameFontNormal")
	bossSelectLabel:SetSize(unpack(brhT.graphics.bossCheckBoxes.bossSelectLabelSize))
	bossSelectLabel:SetPoint("BOTTOM", bossCheckBoxes[1][1], "TOP", unpack(brhT.graphics.bossCheckBoxes.bossSelectLabelPoint))
	bossSelectLabel:SetText(brhT.text.bossSelectLabel)
	bossSelectLabel:SetFont("Fonts\\FRIZQT__.TTF", 16, "")
	
--Sets up calculate Button
	calcButton = CreateFrame("Button", addonName.."CalcButton", mainFrame, "UIPanelButtonTemplate")
	calcButton:SetSize(unpack(brhT.graphics.calcButton.size))
	calcButton:SetPoint("TOPRIGHT", mainFrame, "TOPRIGHT", unpack(brhT.graphics.calcButton.point))
	calcButton:SetText(brhT.text.calcButton.text)
	calcButton:SetScript("OnClick", 
		function(self)
			if not IsAddOnLoaded("Blizzard_EncounterJournal") then
				local loaded, reason = LoadAddOn("Blizzard_EncounterJournal")
				if not loaded then
					print(brhT.text.error.EJNotLoaded:format(tostring(reason)))
				end
			end
			local numSelectedBosses = 0
			for i, boss in ipairs(brhV.bosses) do
				numSelectedBosses = numSelectedBosses + (boss.diff > 0 and 1 or 0)
			end
			
			if brhT.calc() == false then
				return false
			end
			setCalculatedBossInfo()
			if BiSOutput:IsShown() then
				BiSButton:Click()
				if not biSIsUpdate then
					BiSButton:Click()
				end
			end
		end)
	--tooltip
	calcButton:SetScript("OnEnter",
	function(self)
		GameTooltip:SetOwner(self, "ANCHOR_TOPLEFT")
		GameTooltip:SetText(brhT.text.calcButton.tooltip, nil, nil, nil, nil, true)
		if not brhV.options.showTooltip then GameTooltip:Hide() end
	end)
	calcButton:SetScript("OnLeave", function(self) GameTooltip:Hide() end)
	
--Sets up show BiS Button
	BiSButton = CreateFrame("Button", addonName.."BiSButton", mainFrame, "UIPanelButtonTemplate")
	BiSButton:SetSize(unpack(brhT.graphics.BiSButton.size))
	BiSButton:SetPoint("LEFT", calcButton, "RIGHT", unpack(brhT.graphics.BiSButton.point))
	BiSButton:SetText(brhT.text.BiSButton.textShow)
	BiSButton:SetScript("OnClick",
	function(self)
		if BiSOutput:IsShown() then
			BiSOutput:Hide()
			BiSButton:SetText(brhT.text.BiSButton.textShow)
			setCalculatedBossInfo()
		elseif not brhV.BiS[1] then
			print(brhT.text.error.noBiSList)
		else
			BiSOutput:Show()
			BiSOutput.EditBox:SetText(brhT.printBiS())
			BiSButton:SetText(brhT.text.BiSButton.textHide)
			for i, button in ipairs(resultButton) do
				button:Hide()
			end 
		end
	end)
	--tooltip
	BiSButton:SetScript("OnEnter",
	function(self)
		GameTooltip:SetOwner(self, "ANCHOR_TOPLEFT")
		GameTooltip:SetText(brhT.text.BiSButton.tooltip, nil, nil, nil, nil, true)
		if not brhV.options.showTooltip then GameTooltip:Hide() end
	end)
	BiSButton:SetScript("OnLeave", function(self) GameTooltip:Hide() end)

--Sets up the BiS output frame
	BiSOutput = CreateFrame("ScrollFrame", addonName.."OutputFrame", mainFrame, "InputScrollFrameTemplate")
	BiSOutput:SetSize(unpack(brhT.graphics.BiSOutput.size))
 	BiSOutput:SetPoint("TOPRIGHT", mainFrame, "TOPRIGHT", unpack(brhT.graphics.BiSOutput.point))
 	BiSOutput.EditBox:SetText("")
	BiSOutput.EditBox:SetFont("Fonts\\FRIZQT__.TTF", 14)
	BiSOutput.EditBox:SetMaxLetters(0)
	BiSOutput.EditBox:Enable()
	BiSOutput.EditBox:EnableMouse(true)
	BiSOutput.EditBox:SetWidth(BiSOutput:GetWidth())
	BiSOutput.EditBox:SetScript("OnEnterPressed",
		function(self)
			self:HighlightText(0, 0)
			self:ClearFocus()
		end)
	BiSOutput.EditBox:SetScript("OnEscapePressed",
		function(self)
			self:HighlightText(0, 0)
			self:ClearFocus()
		end)
	BiSOutput.EditBox:SetScript("OnLeave",
		function(self)
			self:HighlightText(0, 0)
			self:ClearFocus()
		end)
	BiSOutput:SetFrameLevel(5)
	BiSOutput.CharCount:Hide()
	BiSOutput:EnableMouse(false)
	BiSOutput:Hide()

--Sets up the boss frames
	for i = 1, 8, 1 do
		resultButton[i] = CreateFrame("BUTTON", addonName.."ResultButton"..i, mainFrame, "EncounterBossButtonTemplate")
		currButton = resultButton[i]
		if i > 1 then
			currButton:SetPoint("TOPLEFT", resultButton[i-1], "BOTTOMLEFT", unpack(brhT.graphics.bossFrames.pointGeneric))
		else
			currButton:SetPoint("TOPLEFT", calcButton, "BOTTOMLEFT", unpack(brhT.graphics.bossFrames.pointInit))
		end
		currButton:Show()
		currButton:SetScript("OnClick",
			function(self)
				EncounterJournal_OpenJournal(self.boss.diff > 14 and self.boss.diff or 14, brhT.raidID, self.boss.ID, nil, nil, true)
			end)
		currButton.index = i
		currButton.boss = brhT.newBoss()
		--tooltip
		currButton:SetScript("OnEnter",
			function(self)
				GameTooltip:SetOwner(self, "ANCHOR_TOPLEFT")
				local tooltipText = ""
				if brhV.options.showTooltip then
					tooltipText = brhT.text.resultButtons.tooltipUpper:format(brhT.text.resultButtons.wordFromIndex[self.index])
				end
				if brhV.options.showTooltip and brhV.bosses[self.index] then
					tooltipText = tooltipText.."\n"
				end
				if self.boss and brhV.bosses[1] then
					tooltipText = tooltipText..brhT.text.resultButtons.tooltipLower:format(self.boss.score / (resultButton[1].boss.score == 0 and 1 or resultButton[1].boss.score) * 100)
				end
				GameTooltip:SetText(tooltipText, nil, nil, nil, nil, true)
				if tooltipText == "" then GameTooltip:Hide() end
			end)
		currButton:SetScript("OnLeave", function(self) GameTooltip:Hide() end)
	end
	if brhV.bosses[1] then
		setCalculatedBossInfo()
	end

--Sets up the BonusScore Dropdown
	bonusScoreSelect = CreateFrame("Button", addonName.."BonusScoreSelect", mainFrame, "UIDropDownMenuTemplate")
	bonusScoreSelect:SetPoint("TOPLEFT", mainFrame, "TOPLEFT", unpack(brhT.graphics.bonusScoreSelect.point))
	UIDropDownMenu_SetWidth(bonusScoreSelect, brhT.graphics.bonusScoreSelect.width)
	UIDropDownMenu_JustifyText(bonusScoreSelect, "LEFT")
	UIDropDownMenu_SetText(bonusScoreSelect, brhT.text.bonusScoreSelect.text)
	UIDropDownMenu_Initialize(bonusScoreSelect,
		function(frame, level, menuList)
			local info = UIDropDownMenu_CreateInfo()
			info.text = brhT.text.bonusScoreSelect.newEntry
			info.func = 
			function(self, arg1, arg2, checked)
				UIDropDownMenu_SetText(bonusScoreSelect, brhT.text.bonusScoreSelect.text)
				CloseDropDownMenus()
				bonusScoreValue:SetText(brhT.text.bonusScoreValue.text)
				bonusScoreNew:Show()
				bonusScoreSelect:Hide()
			end
			UIDropDownMenu_AddButton(info)
			for name, bonusScore in pairs(brhV.bonusScores) do
				info = UIDropDownMenu_CreateInfo()
				info.text, info.arg1 = name, name
				info.func =
				function(self, arg1, arg2, checked)
					UIDropDownMenu_SetText(bonusScoreSelect, arg1)
					bonusScoreCurrentName = arg1
					bonusScoreValue:SetText(brhV.bonusScores[arg1].text or "")
					CloseDropDownMenus()
				end
				UIDropDownMenu_AddButton(info)
			end
		end)
	--tooltip
	bonusScoreSelect:SetScript("OnEnter",
	function(self)
		GameTooltip:SetOwner(self, "ANCHOR_TOPLEFT")
		GameTooltip:SetText(brhT.text.bonusScoreSelect.tooltip, nil, nil, nil, nil, true)
		if not brhV.options.showTooltip then GameTooltip:Hide() end
	end)
	bonusScoreSelect:SetScript("OnLeave", function(self) GameTooltip:Hide() end)

--Sets up the bonusScoreNew Editbox
	bonusScoreNew = CreateFrame("EditBox", addonName.."BonusScoreNew", mainFrame, "InputBoxTemplate")
	bonusScoreNew:SetPoint("TOPLEFT", bonusScoreSelect, "TOPLEFT", unpack(brhT.graphics.bonusScoreNew.point))
	bonusScoreNew:SetSize(unpack(brhT.graphics.bonusScoreNew.size))
	bonusScoreNew:SetFontObject(GameFontNormal)
	bonusScoreNew:SetText(brhT.text.bonusScoreSelect.new)
	bonusScoreNew:SetAutoFocus(true)
	local function setNewBonusScore(self)
		bonusScoreCurrentName = self:GetText()

		if not brhV.bonusScores[bonusScoreCurrentName] then
			brhV.bonusScores[bonusScoreCurrentName] = {}
		end

		UIDropDownMenu_SetText(bonusScoreSelect, bonusScoreCurrentName)
		self:HighlightText(0,0)
		self:ClearFocus()
		bonusScoreNew:SetText(brhT.text.bonusScoreSelect.new)
		bonusScoreValue:SetText(brhV.bonusScores[bonusScoreCurrentName].text or brhT.text.bonusScoreValue.text)
		bonusScoreNew:Hide()
		bonusScoreSelect:Show()
		bonusScoreValue:SetFocus()
		bonusScoreValue:HighlightText()
	end
	bonusScoreNew:SetScript("OnEnterPressed", setNewBonusScore)
	bonusScoreNew:SetScript("OnEditFocusLost", setNewBonusScore)
	bonusScoreNew:SetScript("OnTabPressed", setNewBonusScore)
	bonusScoreNew:Hide()

--Sets up the BonusScoreValue EditBox
	bonusScoreValue = CreateFrame("EditBox", addonName.."bonusScoreValue", mainFrame, "InputBoxTemplate")
	bonusScoreValue:SetPoint("TOPLEFT", bonusScoreSelect, "BOTTOMLEFT", unpack(brhT.graphics.bonusScoreValue.point))
	bonusScoreValue:SetSize(unpack(brhT.graphics.bonusScoreValue.size))
	bonusScoreValue:SetFontObject(GameFontNormal)
	bonusScoreValue:SetText(brhT.text.bonusScoreValue.text)
	bonusScoreValue:SetAutoFocus(false)

	--tooltip
	local shortStatNamesList = ""
	for i, stat in ipairs(brhT.stats) do
		shortStatNamesList = shortStatNamesList..stat.nameShort..", "
	end
	shortStatNamesList = shortStatNamesList:sub(1, -2)
	bonusScoreValue:SetScript("OnEnter",
	function(self)
		GameTooltip:SetOwner(self, "ANCHOR_TOPLEFT")
		GameTooltip:SetText(brhT.text.bonusScoreValue.tooltip:format(shortStatNamesList, brhT.stats[10].nameShort, brhT.stats[5].nameShort, brhT.stats[7].nameShort), nil, nil, nil, nil, true)
		if not brhV.options.showTooltip then GameTooltip:Hide() end
	end)
	bonusScoreValue:SetScript("OnLeave", function(self) GameTooltip:Hide() end)


	bonusScoreValue:SetScript("OnEnterPressed", 
		function(self)
			local text = self:GetText()
			self:HighlightText(0,0)
			self:ClearFocus()
			if not bonusScoreCurrentName then
				return
			end
			--clears everything when value is empty or zero
			if tonumber(text) == 0 or text:find("^%s+$") then
				brhV.bonusScores[bonusScoreCurrentName] = nil
				UIDropDownMenu_SetText(bonusScoreSelect, brhT.text.bonusScoreSelect.text)
				bonusScoreValue:SetText(brhT.text.bonusScoreValue.text)
				return
			end

			brhV.bonusScores[bonusScoreCurrentName].text = text

			--creates function from input
			local func, err = "return ", false
			local op = "[%+%-%*%/]"	--list of operands
			local lastStrOp = true
		
			--cleans input
			text = text:gsub("%s+", "")
			text = text:gsub(op, " %1 ")
		
			for str in text:gmatch("%S+") do
				local currStrOp = str:find(op)

				if lastStrOp and currStrOp then
					func, err =  "", brhT.text.error.bonusScoreFuncop
					break
				end

				if currStrOp or str:find("^%d+%.?%d*$") then
					func = func..str
				elseif str:find("^%l+$") then
					if brhT.stats[str] then
							func = func.."brhV.statweights[brhV.playerInfo.specID][\""..brhT.stats[str].variable.."\"]"
					else
						func, err = "", brhT.text.error.bonusScoreFunc.statName
						break
					end
				else
					func, err =  "", ""
					break
				end

				lastStrOp = currStrOp
			end

			if err then
				print(brhT.text.error.bonusScoreFunc.error:format(err))
				brhV.bonusScores[bonusScoreCurrentName].func = 
					function()
						print(brhT.text.error.bonusScoreFunc.errorFunc:format(bonusScoreCurrentName))
						return 0
					end
			else
				brhV.bonusScores[bonusScoreCurrentName].func = func
				UIDropDownMenu_SetText(bonusScoreSelect, brhT.text.bonusScoreSelect.text)
				bonusScoreValue:SetText(brhT.text.bonusScoreValue.text)
			end	
		end)
--Sets up the options Checkboxes
	for i, option in ipairs(brhT.options) do

		optionBoxes[i] = CreateFrame("CheckButton", addonName.."OptionBox"..i, mainFrame, "UICheckButtonTemplate")
		local currBox = optionBoxes[i]
		currBox:SetSize(unpack(brhT.graphics.options.size))
		if i == 1 then
			currBox:SetPoint("BOTTOMLEFT", mainFrame, "BOTTOMLEFT", unpack(brhT.graphics.options.pointInit))
		else
			currBox:SetPoint("BOTTOMLEFT", optionBoxes[i - 1], "TOPLEFT", unpack(brhT.graphics.options.pointGeneric))
		end
		local currLabel = _G[currBox:GetName().."Text"]
		currLabel:ClearAllPoints()
		currLabel:SetPoint("TOPLEFT", currBox, "TOPRIGHT", unpack(brhT.graphics.options.labelPoint))
		currLabel:SetJustifyH("LEFT")
		currLabel:SetText(option.text)
		currLabel:SetSize(unpack(brhT.graphics.options.labelSize))
		currBox:SetScript("OnClick", option.func)
		currBox:SetScript("OnEnter",
			function(self)
				GameTooltip:SetOwner(self, "ANCHOR_TOPLEFT")
				GameTooltip:SetText(option.tooltip, nil, nil, nil, nil, true)
				if not brhV.options.showTooltip then
					GameTooltip:Hide()
				end
			end)
		currBox:SetScript("OnLeave",
		function()
			GameTooltip:Hide()
		end)
		if brhV.options[option.name] then
			currBox:Click("LeftButton")
		else
			currBox:Click("LeftButton")
			currBox:Click("LeftButton")
		end
	end

end

eventFrame = CreateFrame("Frame", addonName.."EventFrame", UIParent)
eventFrame:SetSize(1, 1)
eventFrame:SetPoint("TOPLEFT", UIParent, "TOPLEFT")
eventFrame:EnableMouse(false)
eventFrame:EnableKeyboard(false)
eventFrame:SetMovable(false)
eventFrame:RegisterEvent("SPELL_CONFIRMATION_PROMPT")
eventFrame:RegisterEvent("PLAYER_ENTERING_WORLD")
eventFrame:RegisterEvent("PLAYER_SPECIALIZATION_CHANGED")
eventFrame:RegisterEvent("ADDON_LOADED")
local function eventFrameOnEvent(self, event, ...)
	if event == "SPELL_CONFIRMATION_PROMPT" then
		if not (brhV.bosses and brhV.bosses[1]) or not brhV.options.hideBonusRollWindow then
			return
		end
		local spellID, confirmType, text, duration, currencyID, difficultyID = ...
		if confirmType ~= LE_SPELL_CONFIRMATION_PROMPT_TYPE_BONUS_ROLL then
			return
		end
		local instanceID, encounterID = GetJournalInfoForSpellConfirmation(spellID)
		local currBoss
		for i = 1, brhT.numBosses, 1 do
			if encounterID and brhV.bosses[1].ID == encounterID then
				currBoss = brhV.bosses[1]
			end
		end
		if not(currBoss and currBoss.diff == difficultyID and currBoss.shouldCoin) then 
			BonusRollFrame:Hide()
		end
	--[[elseif event == "PLAYER_ENTERING_WORLD" then
		for _, info in pairs(GetSpellConfirmationPromptsInfo()) do
			eventFrameOnEvent(eventFrame, "SPELL_CONFIRMATION_PROMPT", info.spellID, info.confirmType, info.text, info.duration, info.currencyID, info.difficultyID)
		end]]
	elseif event == "PLAYER_SPECIALIZATION_CHANGED" and ... == "player" then
		if brhT.frameCreated and brhV.playerInfo.specID ~= GetSpecializationInfo(GetSpecialization()) then
			brhT.getPlayerInfo()
			for i, currBox in ipairs(statWeightBoxes) do
				currBox:SetText(brhV.statweights[brhV.playerInfo.specID][currBox.stat.variable] or 0)
			end
			if mainFrame:IsShown() then
				optionBoxes[1]:Click()
				optionBoxes[1]:Click()
			end
		end
	elseif event == "ADDON_LOADED" and ... == addonName then
		setmetatable(brhV.statweights, brhT.statweightsMT)
		
		if type(brhV.version) ~= "table" or brhV.version.major ~= brhT.version.major or brhV.version.minor ~= brhT.version.minor then
			updateVariables()
		end

		
	end
end
eventFrame:SetScript("OnEvent", eventFrameOnEvent)