local addonName, addonTable = ...

brhV = {} --This WILL be saved between sessions
brhT = {} --This WILL NOT be saved between sessions

--[[
UPDATE EACH TIER:
brhT.raidID
brhT.numBosses
brhT.sealCurrencyID
calc local gem info
]]--

brhT.createVariables = function()
	brhV.statweights = {}				--Saves the selected statweights
	brhV.BiS = {}
	brhV.bonusScores = {}
	brhV.options = {}
	for i, option in ipairs(brhT.options) do
		brhV.options[option.name] = option.default
	end
	brhV.version = {
		major = 0,
		minor = 0
	}
	brhV.playerInfo = {
		specID = 0,
		specName = "",
		classID = 0,
		className = "",
		englishClassName = ""
	}
	setmetatable(brhV.statweights, brhT.statweightsMT)
	brhV.bosses = {}
end


brhT.raidID = 1031						--RaidID of the current Raid (Uldir)
brhT.numBosses = 8						--Number of Bosses of the current Raid (Uldir)
brhT.sealCurrencyID = 1580				--CurrencyID of the bonus roll coins (BfA)
brhT.frameCreated = false				--Used to determine if mainFrame was already created this session
brhT.version = {
	major = 0,
	minor = 1
}
brhT.didVersionChange = false
brhT.statweightsMT = {
	__index =
	function(Table, key)
		Table[key] = {}
		for i, stat in ipairs(brhT.stats) do
			Table[key][stat.variable] = 0
		end
		return Table[key]
	end
}
brhT.getPlayerInfo =
	function()
		brhV.playerInfo.className,
		brhV.playerInfo.englishClassName,
		brhV.playerInfo.classID
		= UnitClass("player")
		if GetSpecializationInfo(GetSpecialization()) then
			brhV.playerInfo.specID,
			brhV.playerInfo.specName
			= GetSpecializationInfo(GetSpecialization())
		end
		local azeriteItemLoc = C_AzeriteItem.FindActiveAzeriteItem()
		brhV.playerInfo.currentAzeriteLevel = azeriteItemLoc and C_AzeriteItem.GetPowerLevel(azeriteItemLoc) or 0
end
brhT.newBoss =
	function()
		local newBoss = {
			index = 0,
			name = "",
			link = "",
			ID = 0,

			score = -1,
			rank = 0,
			numLoot = 0,
			diff = 0,
			isKilled = false,
			shouldCoin = false,
			loot = {}
		}
		return newBoss	
	end
brhT.stats = {
	{
		name = ITEM_MOD_STRENGTH_SHORT,
		nameShort = "str",
		variable = "ITEM_MOD_STRENGTH_SHORT",
		index = 1
	},
	{
		name = ITEM_MOD_AGILITY_SHORT,
		nameShort = "agi",
		variable = "ITEM_MOD_AGILITY_SHORT",
		index = 2
	},
	{
		name = ITEM_MOD_INTELLECT_SHORT,
		nameShort = "int",
		variable = "ITEM_MOD_INTELLECT_SHORT",
		index = 3

	},
	{
		name = ITEM_MOD_STAMINA_SHORT,
		nameShort = "stam",
		variable = "ITEM_MOD_STAMINA_SHORT",
		index = 4
	},
	{
		name = RESISTANCE0_NAME,
		nameShort = "armor",
		variable = "RESISTANCE0_NAME",
		index = 5
	},
	{
		name = ITEM_MOD_HASTE_RATING_SHORT,
		nameShort = "haste",
		variable = "ITEM_MOD_HASTE_RATING_SHORT",
		index = 6
	},
	{
		name = ITEM_MOD_VERSATILITY,
		nameShort = "vers",
		variable = "ITEM_MOD_VERSATILITY",
		index = 7
	},
	{
		name = ITEM_MOD_CRIT_RATING_SHORT,
		nameShort = "crit",
		variable = "ITEM_MOD_CRIT_RATING_SHORT",
		index = 8
	},
	{
		name = ITEM_MOD_MASTERY_RATING_SHORT,
		nameShort = "mastery",
		variable = "ITEM_MOD_MASTERY_RATING_SHORT",
		index = 9
	},
	{
		name = ITEM_MOD_CR_AVOIDANCE_SHORT,
		nameShort = "avoidance",
		variable = "ITEM_MOD_CR_AVOIDANCE_SHORT",
		index = 10
	},
	{
		name = ITEM_MOD_CR_LIFESTEAL_SHORT,
		nameShort = "leech",
		variable = "ITEM_MOD_CR_LIFESTEAL_SHORT",
		index = 11
	},
	{
		name = "Weapon DPS",
		nameShort = "weapondps",
		variable = "ITEM_MOD_DAMAGE_PER_SECOND_SHORT",
		index = 12
	}
}
for i, stat in ipairs(brhT.stats) do
	brhT.stats[stat.name] = brhT.stats[i]
	brhT.stats[stat.nameShort] = brhT.stats[i]
	brhT.stats[stat.variable] = brhT.stats[i]
end

brhT.options = {
	{
		name = "compactStatweights",
		text = "Compact statweights",
		tooltip = "If checked, statweights that are zero are hidden.",
		default = false,
		func =
			function(self)
				local isChecked = false
				if type(self) == "table" then
					isChecked = self:GetChecked()
				else
					isChecked = self
				end

				if isChecked then
					brhV.options.compactStatweights = true
					local lastActiveIndex =  0
					for i, currBox in ipairs(brhT.statWeightBoxes) do
						if currBox:GetNumber() == 0 then
							currBox:Hide()
						else
							currBox:ClearAllPoints()
							if lastActiveIndex == 0 then
								currBox:SetPoint("TOPLEFT", _G[addonName.."Main"], "TOPLEFT", unpack(brhT.graphics.statWeightBoxes.pointInit))
							else
								currBox:SetPoint("TOPLEFT", brhT.statWeightBoxes[lastActiveIndex], "TOPLEFT", unpack(brhT.graphics.statWeightBoxes.pointGeneric))
							end
							lastActiveIndex = i
						end
					end
					_G[addonName.."pawnImportButton"]:SetPoint("TOPRIGHT", brhT.statWeightBoxes[lastActiveIndex], "TOPRIGHT", unpack(brhT.graphics.pawnImportButton.point))
				else
					brhV.options.compactStatweights = false
					for i, currBox in ipairs(brhT.statWeightBoxes) do
						currBox:Show()
						currBox:ClearAllPoints()
						if i == 1 then
							currBox:SetPoint("TOPLEFT", _G[addonName.."Main"], "TOPLEFT", unpack(brhT.graphics.statWeightBoxes.pointInit))
						else
							currBox:SetPoint("TOPLEFT", brhT.statWeightBoxes[i - 1], "TOPLEFT", unpack(brhT.graphics.statWeightBoxes.pointGeneric))
						end
					end
					_G[addonName.."pawnImportButton"]:SetPoint("TOPRIGHT", brhT.statWeightBoxes[#brhT.statWeightBoxes], "TOPRIGHT", unpack(brhT.graphics.pawnImportButton.point))
				end
			end
	},
	{
		name = "showTooltip",
		text = "Show Tooltips",
		tooltip = "If checked, all explanatory tooltips for this addon are hidden.",
		default = true,
		func =
			function(self)
				brhV.options.showTooltip = self:GetChecked() or false
			end
	},
	{
		name = "hideBonusRollWindow",
		text = "Hide bad bonus roll winodws",
		tooltip = "If checked, the frame allowing you to use your coins will be hidden on all bosses apart from the ones calculated here.\nIf you still want to use a coin on a different boss type \"/brh show\" after killing a boss (without \"\").",
		default = false,
		func =
			function(self)
				brhV.options.hideBonusRollWindow = self:GetChecked() or false
			end
	},
	{
		name = "useCurrentAzeriteLevel",
		text = "Use current Azerite level",
		tooltip = "Determines whether an item's score should include all Azerite powers or only ones currently unlockable.",
		default = false,
		func =
			function(self)
				brhV.options.useCurrentAzeriteLevel = self:GetChecked() or false
			end
	}
}
brhT.currChangeInfo = {}
brhT.text = {
	error = {
		noBonusRollsHidden = "There currently are no bonus rolls that are being hidden by this addon.",
		unknownSlashCommand = "Unknown command. Available commands are:\n\"/brh\" for opening the main window\n\"/brh show\" for showing the bonus roll frame",
		noBiSList = "You need to calculate your BiS list first.",
		EJNotLoaded = "ERROR: EJ not loaded. Reason: %s",
		bonusScoreFunc = {
			op = "Two operators next to each other/operator at start",
			statName = "Incorrect stat name",
			error = "ERROR: Bonus Score formula not correct. %s",
			errorFunc = "The item/power %q has an incorrect bonus score formula!"
		},
		unknownStat = "%s has an unknmown stat: %s\n please submit a bug report or comment on Curse with a screenshot of this message",
		notLoaded = "WoW has not finished loading, please wait a bit and try again."
	},
	updateMessage = "Updates:\n",
	changeInfo = {
		noVersion = "Due to an update all saved data has been reset.",
		[0] = {
			[1] = "Full weapon support\nSocket stat has been removed, extra stats from gems are now added to that item's stats automatically\nStatweights can be imported from Pawn strings\nClicking a difficulty name will set all bosses to that difficulty"
		}
	},
	BiSButton = {
		textHide = "Hide BiS list",
		textShow = "Show BiS list",
		tooltip = "Shows a BiS list you can copy-paste out of.\nAny changes you make to the list are not saved and have no effect.\nGreen items are already equipped, purple ones are loot.",
		hideUpdateText = "Hide updates",
		hideUpdateTooltip = ""
	},
	statWeightBoxes = {
		text = 0,
		label = "Stat Weights:"
	},
	bossSelectLabel = "Select Bosses:",
	calcButton = {
		text = "Calculate",
		tooltip = "Calculates scores for all selected boss loot and equipped items."
	},
	resultButtons = {
		wordFromIndex = {"", "second ", "third ", "fourth ", "fifth ", "sixth ", "seventh ", "eigth "},
		tooltipUpper = "This is the boss with the %s best chance for an upgrade",
		tooltipLower = "Relative score: %i%%"
	},
	bonusScoreSelect = {
		text = "Select item or power",
		newEntry = "NEW ENTRY",
		tooltip = "You can select an item or azerite power name here and then enter a number or formula below to give it a bonus to its score.\nUseful for trinket abilities, azerite powers, etc.\nYou can delete names from this list by giving it an empty or 0 bonus score.",
		new = "New name"
	},
	bonusScoreValue = {
		text = "Bonus score",
		tooltip = "Enter a number or formula indicating the selected items or powers bonus score here.\nIf you want to enter a formula, it has to have a very specific form!\nAllowed operators are + - * / \nAllowed statnames are\n%s\nAllowed inputs are e.g.\n500\n8.5 * %s\n50*%s+250*%s"
	},
	BiSOutput = {
		colorLoot = select(4, GetItemQualityColor(4)),
		colorEquipped = select(4, GetItemQualityColor(2)),
		lineLoot = "%s: |c%s %s |r (%s)\n",
		lineEquipped = "%s:|c%s %s |r \n"
	},
	pawnImportButton = {
		text = "Import Pawn string",
		tooltip = "Import statweights from Pawn, Simcraft, Raidbots, etc."
	},
	pawnImpotEditbox = {
		text = ""
	}
}
brhT.graphics = {
	mainFrame = {
		size = {1050, 650},
		point = {0, 100}
	},
	closeButton = {
		size = {30, 30},
		point = {-3, -3}
	},
	statWeightBoxes = {
		size = {50, 35},
		pointInit = {150, -45},
		pointGeneric = {0, -30},
		labelSize = {90, 35},
		labelPoint = {-5, 0},
		statWeightsLabelSize = {150, 35},
		statWeightsLabelPoint = {0, -10}
	},
	bossCheckBoxes = {
		size = {30, 30},
		pointInit = {430, -75},
		pointInitRow = {0, -30},
		pointGeneric = {40, 0},
		labelSize = {150, 30},
		labelPoint = {0, 0},
		diffLabelPoint = {0, 5},
		diffLabelSize = {60, 13},
		bossSelectLabelPoint = {0, 30},
		bossSelectLabelSize = {120, 13}
	},
	calcButton = {
		size = {100, 30},
		point = {-325, -25}
	},
	BiSButton= {
		size = {100, 30},
		point = {30, 0}
	},
	BiSOutput = {
		size = {450, 300},
		point = {-25, -70}
	},
	bossFrames = {
		pointInit = {0, -10},
		pointGeneric = {0, -10}
	},
	bonusScoreSelect = {
		width = 150,
		point = {355, -460}
	},
	bonusScoreNew = {
		size = {162, 35},
		point = {20, 5}
	},
	bonusScoreValue = {
		size = {162, 35},
		point = {20, 0}
	},
	options = {
		pointInit = {35, 30},
		pointGeneric = {0, 0},
		size = {25, 25},
		labelPoint = {0, 0},
		labelSize = {200, 25}
	},
	pawnImportButton = {
		size = {150, 25},
		point = {0, -40}
	},
	pawnImpotEditbox = {
		size = {142, 35},
		point = {5, 5}
	}
}
brhT.createVariables()